import pdb
import pandas as pd
from pyecharts.charts import Page
from prettytable import PrettyTable
from pyecharts.charts import TreeMap
from pyecharts.components import Table
from pyecharts.charts import Tab
from pyecharts.options import ComponentTitleOpts
from pyecharts import options as opts
from pyecharts.types import Optional, Sequence

from .sentiment import Sentiment as plot_stock_sentiment
from .flow_money import FlowMoney as plot_stock_flow_money
from .hkshshz import HkShSz as plot_stock_hkshsz

import html

class PosfeatTable(Table):
    def add(self, headers: Sequence, rows:Sequence, attributes: Optional[dict]=None):
        attributes = {"style":"font-size: 13px;"}
        table = PrettyTable(headers, attributes=attributes)
        for r in rows:
            table.add_row(r)
        self.html_content = html.unescape(table.get_html_string(format=True))
        return self
    
class ThemeTable(Table):
    def add(self, headers: Sequence, rows:Sequence, attributes: Optional[dict]=None):
        attributes = {"style":"font-size: 13px;"}
        table = PrettyTable(headers, attributes=attributes)
        for r in rows:
            table.add_row(r)
        table.align["驱动事件"] = 'l'
        table.align["主题概念"] = 'l'
        self.html_content = html.unescape(table.get_html_string(format=True))
        return self
    

class Detail(object):
    @classmethod
    def draw(cls, name, sentiment_data, flowmoeny_data, hkshsz_data, theme_data, posfeat_data):
        page = Page(layout=Page.DraggablePageLayout)
        page.add(
            cls.theme_table(theme_data, name),
            plot_stock_sentiment.sentiment(sentiment_data, name,'1200px', '900px'),
            cls.posfeat(posfeat_data, name),
            plot_stock_flow_money.flow_money(flowmoeny_data, name,'1200px', '900px'),
            plot_stock_hkshsz.hkshsz(hkshsz_data, name,'1200px', '900px'),
        )
        return page
    
    @classmethod
    def posfeat(cls, data, name):
        period_dict = {'1b':'最近1天','3b':'最近3天','5b':'最近5天'}
        table = PosfeatTable()
        columns = ['period','cumsum_buy','cumsum_sell','cumsum_net','mechanism_buy','mechanism_sell',
                  'mechanism_net','mechanism_position','capital_buy','capital_sell','capital_net',
                  'capital_position','capital_name']
        data = data[columns]
        if not data.empty:
            data['period'] = data['period'].apply(lambda x: period_dict[x])
            data['capital_position'] = data['capital_position'].round(4).apply(lambda x: str(x * 100)[0:4] + '%')
            data['mechanism_position'] = data['mechanism_position'].round(4).apply(lambda x: str(x * 100)[0:4] + '%')
        data = data.rename(columns={'period':'周期','cumsum_buy':'累计买入','cumsum_sell':'累计卖出','cumsum_net':'累计净买入',
                                'mechanism_buy':'机构买入','mechanism_sell':'机构卖出','mechanism_net':'机构净买入',
                                'mechanism_position':'机构仓位',
                                'capital_buy':'游资买入','capital_sell':'游资卖出','capital_net':'游资净买入',
                                'capital_position':'游资仓位','capital_name':'参与游资'})
        headers = data.columns.tolist()
        rows = data.values.tolist()
        table.add(headers, rows)
        table.set_global_opts(
            title_opts=opts.ComponentTitleOpts(title= name + " 仓位信息")
            
        )
        return table
        
    @classmethod
    def theme_table(cls, data, name):
        table = ThemeTable()
        columns = ['tid','name','event','update_time']
        if not data.empty:
            data = data[columns]
            data['update_time'] = data['update_time'].dt.strftime('%Y-%m-%d')
            data['event'] = data['event'].apply(lambda x: x[:28] + '......' if len(x) > 28 else x)
            data['name'] = "<a href='../detail/" + data['tid'] + ".html'>" + data['name'] + "</a>"
            data = data.sort_values(by='update_time', ascending=False).drop(['tid'],axis=1).rename(
                columns={'name':'主题概念','update_time':'更新时间','event':'驱动事件'}).head(20)
        headers = data.columns.tolist()
        rows = data.values.tolist()
        table.add(headers, rows)
        table.set_global_opts(
            title_opts=opts.ComponentTitleOpts(title= name + " 关联主题概念")
            
        )
        return table